#include "dumpacct.h"

/*
 * dumpacct.c
 *
 * program that dumps the contents of an account
 */

void main(int argc, char *argv[])
{
  int amount, md_counter, d_counter, start_date, end_date;
  char *acctnum, *analnum, *acctnam;
  char minibuff[513];
  char *pgDatum;

  pgDatum = malloc(11);

  if (argc < 5) {
    fprintf(stderr, "Usage: %s acct anal start end\n", argv[0]);
    fprintf(stderr, "where acct is three-digit account number\n");
    fprintf(stderr, "      anal is two-digit analytical extension\n");
    fprintf(stderr, "      start is start date in format YYYYMMDD\n");
    fprintf(stderr, "      end is end date in format YYYYMMDD\n");
    exit(0);
  }

  /* check validity of supplied account number and then copy it to 
   * the local buffers acctnum and analnum so that if no analytical
   * number is supplied, analnum carries the default value of "--"
   */
  if ((strlen(argv[1]) != 3) ||
      ((argc > 2) && (strlen(argv[2]) != 2))) {
    fprintf(stderr, "Invalid account number.\n");
    exit(0);
  }
  acctnum = malloc(4);
  analnum = malloc(3);
  acctnum = strcpy(acctnum, argv[1]);
  if (argc > 2)
    analnum = strcpy(analnum, argv[2]);
  else analnum = "--";

  /* check validity of dates
   */
  start_date = atoi(argv[3]);
  end_date = atoi(argv[4]);
  if ((!valiDate(start_date)) ||
      (!valiDate(end_date))) {
    fprintf(stderr, "Dates must be in format YYYYMMDD\n");
    exit(0);
  }
  if (start_date > end_date) {
    fprintf(stderr, "Start date must be less than or equal to end date!\n");
    exit(0);
  }

  initBackend();                /* initialize the backend connection */

  acctnam = malloc(61);
  if ((acctnam = acctQuery(acctnam, acctnum, analnum)) == NULL) {
    fprintf(stderr, "The account you specified is not in the chart of accounts.\n");
    exit_nicely();
  } else {
    printf("%s %s: %s\n", acctnum, analnum, acctnam);
  }

  md_counter = d_counter = 0;
  sprintf(minibuff, "DECLARE a1 CURSOR FOR SELECT W1.ent_date, W1.por_cislo, W2.desig, W1.md_acct, W1.md_anal, W1.md_amt, W1.d_acct, W1.d_anal, W1.d_amt FROM denik W1, descs W2 WHERE (W1.ent_date>='%s' AND W1.ent_date<='%s') AND W1.ent_date=W2.ent_date AND W1.por_cislo=W2.por_cislo AND ((W1.md_acct='%s' AND W1.md_anal='%s') OR (W1.d_acct='%s' AND W1.d_anal='%s')) ORDER BY ent_date, por_cislo;", pgConvertDate(pgDatum, start_date), pgConvertDate(pgDatum, end_date), acctnum, analnum, acctnum, analnum);
  startTransaction();
  runQuery(minibuff);
  printf("------------+--------------------------------+-------------+-------------\n");
  printf("Datum:PorCis|Popis Transakce                 |   MA DATI   |    DAL\n");
  printf("------------+--------------------------------+-------------+-------------\n");
  while (fetchNext("a1")) {
    printf("%d:%03d ", (NathanConvertDate(getField("ent_date"))), atoi(getField("por_cislo")));
    if ((amount = atoi(getField("md_amt"))) > 0) {
      printf("%-30s  %10d.%02d\n", getField("desig"), (amount/100), (amount%100));
      md_counter = md_counter + amount;
    } else 
      if ((amount = atoi(getField("d_amt"))) > 0) {
	printf("%-30s                %10d.%02d\n", getField("desig"), (amount/100), (amount%100));
	d_counter = d_counter + amount;
      } else {
	fprintf(stderr, "Corrupt database: either credit or debit side must = 0 in 'denik'");
	exit_nicely();
      }
  }
  printf("\nTOTALS:                                       %10d.%02d %10d.%02d\n", (md_counter/100), (md_counter%100), (d_counter/100), (d_counter%100));

  endTransaction();
  
  exit_nicely();
}
      
